@@ -22,7 +22,7 @@ from UM.Settings.Validator import Validator
 
 class DefinitionPropertyType(enum.IntEnum):
     """Type of definition property.
-    
+
     This enum describes the possible types for a supported definition property.
     For more information about supported definition properties see SettingDefinition
     and SettingDefinition::addSupportedProperty().
@@ -56,7 +56,7 @@ def _toFloatConversion(value: str) -> float:
 
 def _toIntConversion(value):
     """Conversion from string to integer.
-    
+
     :param value: The string representation of an integer.
     """
 
@@ -67,20 +67,20 @@ def _toIntConversion(value):
 
 class SettingDefinition:
     """Defines a single Setting with its properties.
-    
+
     This class defines a single Setting with all its properties. This class is considered immutable,
     the only way to change it is using deserialize(). Should any state need to be stored for a definition,
     create a SettingInstance pointing to the definition, then store the value in that instance.
-    
+
     == Supported Properties
-    
+
     The SettingDefinition class contains a concept of "supported properties". These are properties that
     are supported when serializing or deserializing a setting. These properties are defined through the
     addSupportedProperty() method. Each property needs a name and a type. In addition, there are two
     optional boolean value to indicate whether the property is "required" and whether it is "read only".
     Currently, four types of supported properties are defined. Please DefinitionPropertyType for a description
     of these types.
-    
+
     Required properties are properties that should be present when deserializing a setting. If the property
     is not present, an error will be raised. Read-only properties are properties that should never change
     after creating a SettingDefinition. This means they cannot be stored in a SettingInstance object.
@@ -88,7 +88,7 @@ class SettingDefinition:
 
     def __init__(self, key: str, container: Optional[DefinitionContainerInterface] = None, parent: Optional["SettingDefinition"] = None, i18n_catalog: Optional[i18nCatalog] = None) -> None:
         """Construcutor
-        
+
         :param key: :type{string} The unique, machine readable/writable key to use for this setting.
         :param container: :type{DefinitionContainerInterface} The container of this setting. Defaults to None.
         :param parent: :type{SettingDefinition} The parent of this setting. Defaults to None.
@@ -133,9 +133,14 @@ class SettingDefinition:
 
         super().__setattr__(name, value)
 
+    def __hash__(self):
+        """Ensure that the SettingDefinition is hashable, so it can be used in a set."""
+
+        return hash(str(self))
+
     def __getstate__(self):
         """For Pickle support.
-        
+
         This should be identical to Pickle's default behaviour but the default
         behaviour doesn't combine well with a non-default __getattr__.
         """
@@ -144,7 +149,7 @@ class SettingDefinition:
 
     def __setstate__(self, state):
         """For Pickle support.
-        
+
         This should be identical to Pickle's default behaviour but the default
         behaviour doesn't combine well with a non-default __getattr__.
         """
@@ -159,7 +164,7 @@ class SettingDefinition:
     @property
     def key(self) -> str:
         """The key of this setting.
-        
+
         :return: :type{string}
         """
 
@@ -168,7 +173,7 @@ class SettingDefinition:
     @property
     def container(self) -> Optional[DefinitionContainerInterface]:
         """The container of this setting.
-        
+
         :return:
         """
 
@@ -177,7 +182,7 @@ class SettingDefinition:
     @property
     def parent(self) -> Optional["SettingDefinition"]:
         """The parent of this setting.
-        
+
         :return: :type{SettingDefinition}
         """
 
@@ -186,7 +191,7 @@ class SettingDefinition:
     @property
     def children(self) -> List["SettingDefinition"]:
         """A list of children of this setting.
-        
+
         :return: :type{list<SettingDefinition>}
         """
 
@@ -195,7 +200,7 @@ class SettingDefinition:
     @property
     def relations(self) -> List["SettingRelation"]:
         """A list of SettingRelation objects of this setting.
-        
+
         :return: :type{list<SettingRelation>}
         """
 
@@ -203,7 +208,7 @@ class SettingDefinition:
 
     def serialize(self) -> str:
         """Serialize this setting to a string.
-        
+
         :return: :type{string} A serialized representation of this setting.
         """
 
@@ -211,7 +216,7 @@ class SettingDefinition:
 
     def getAllKeys(self) -> Set[str]:
         """Gets the key of this setting definition and of all its descendants.
-        
+
         :return: A set of the key in this definition and all its descendants.
         """
 
@@ -225,7 +230,7 @@ class SettingDefinition:
 
     def serialize_to_dict(self) -> Dict[str, Any]:
         """Serialize this setting to a dict.
-        
+
         :return: :type{dict} A representation of this setting definition.
         """
 
@@ -243,7 +248,7 @@ class SettingDefinition:
 
     def deserialize(self, serialized: Union[str, Dict[str, Any]]) -> None:
         """Deserialize this setting from a string or dict.
-        
+
         :param serialized: :type{string or dict} A serialized representation of this setting.
         """
 
@@ -255,9 +260,9 @@ class SettingDefinition:
 
     def getChild(self, key: str) -> Optional["SettingDefinition"]:
         """Get a child by key
-        
+
         :param key: :type{string} The key of the child to get.
-        
+
         :return: :type{SettingDefinition} The child with the specified key or None if not found.
         """
 
@@ -274,38 +279,53 @@ class SettingDefinition:
 
         return None
 
+    def _matches1l8nProperty(self, property_name: str, value: Any, catalog) -> bool:
+        try:
+            property_value = getattr(self, property_name)
+        except AttributeError:
+            # If we do not have the attribute, we do not match
+            return False
+
+        if catalog:
+            translated_key = "{key} {property_name}".format(key = self._key, property_name = property_name)
+            property_value = catalog.i18nc(translated_key, property_value)
+
+        if not isinstance(value, str):
+            return False
+        if value != property_value:
+            if "*" not in value:
+                return False
+
+            value = value.strip("* ").lower()
+            if value not in property_value.lower():
+                return False
+
+        return True
+
     def matchesFilter(self, **kwargs: Any) -> bool:
         """Check if this setting definition matches the provided criteria.
 
         :param kwargs: :type{dict} A dictionary of keyword arguments that need to match its attributes.
         """
 
-
         # First check for translated labels.
         keywords = kwargs.copy()
         if "i18n_label" in keywords:
-            try:
-                property_value = getattr(self, "label")
-            except AttributeError:
-                # If we do not have the attribute, we do not match
+            if not self._matches1l8nProperty("label", keywords["i18n_label"], keywords.get("i18n_catalog")):
                 return False
-
-            if "i18n_catalog" in keywords:
-                catalog = keywords["i18n_catalog"]
-                if catalog:
-                    property_value = catalog.i18nc(self._key + " label", property_value)
-
-            value = keywords["i18n_label"]
             del keywords["i18n_label"]
-            if not isinstance(value, str):
-                return False
-            if value != property_value:
-                if "*" not in value:
-                    return False
 
-                value = value.strip("* ").lower()
-                if value not in property_value.lower():
+        # There is a special case where we want to filter on either the label and the description.
+        # For the sake of keeping this code simple, I've just hardcoded this option. If we ever want to have multiple
+        # keywords that can be searched as an optional filter (eg; value matches either paramA or paramB, we should
+        # consider refactoring this.
+        # Note that this match will be called a lot, so keep an eye out for performance
+        if "i18n_label|i18n_description" in keywords:
+            matches_label = self._matches1l8nProperty("label", keywords["i18n_label|i18n_description"], keywords.get("i18n_catalog"))
+            if not matches_label:
+                if not self._matches1l8nProperty("description", keywords["i18n_label|i18n_description"], keywords.get("i18n_catalog")):
                     return False
+            del keywords["i18n_label|i18n_description"]
 
         if "i18n_catalog" in keywords:
             del keywords["i18n_catalog"]
@@ -347,17 +367,15 @@ class SettingDefinition:
 
     def findDefinitions(self, **kwargs: Any) -> List["SettingDefinition"]:
         """Find all definitions matching certain criteria.
-        
+
         This will search this definition and its children for definitions matching the search criteria.
-        
+
         :param kwargs: :type{dict} A dictionary of keyword arguments that need to match properties of the children.
-        
+
         :return: :type{list} A list of children matching the search criteria. The list will be empty if no children
         were found.
         """
 
-        definitions = []    # type: List["SettingDefinition"]
-
         if not self.__descendants:
             self.__descendants = self._updateDescendants()
 
@@ -374,6 +392,7 @@ class SettingDefinition:
                     return [self]
                 return [self.__descendants[key]]
 
+        definitions = []  # type: List["SettingDefinition"]
         if self.matchesFilter(**kwargs):
             definitions.append(self)
 
@@ -384,9 +403,9 @@ class SettingDefinition:
 
     def isAncestor(self, key: str) -> bool:
         """Check whether a certain setting is an ancestor of this definition.
-        
+
         :param key: :type{str} The key of the setting to check.
-        
+
         :return: True if the specified setting is an ancestor of this definition, False if not.
         """
 
@@ -397,9 +416,9 @@ class SettingDefinition:
 
     def isDescendant(self, key: str) -> bool:
         """Check whether a certain setting is a descendant of this definition.
-        
+
         :param key: :type{str} The key of the setting to check.
-        
+
         :return: True if the specified setting is a descendant of this definition, False if not.
         """
 
@@ -436,12 +455,12 @@ class SettingDefinition:
     @classmethod
     def addSupportedProperty(cls, name: str, property_type: DefinitionPropertyType, required: bool=False, read_only: bool=False, default: Any=None, depends_on: Optional[str]=None) -> None:
         """Define a new supported property for SettingDefinitions.
-        
+
         Since applications may want custom properties in their definitions, most properties are handled
         dynamically. This allows the application to define what extra properties it wants to support.
         Additionally, it can indicate whether a properties should be considered "required". When a
         required property is not missing during deserialization, an AttributeError will be raised.
-        
+
         :param name: :type{string} The name of the property to define.
         :param property_type: :type{DefinitionPropertyType} The type of property.
         :param kwargs: Keyword arguments. Possible values:
@@ -460,9 +479,9 @@ class SettingDefinition:
     @classmethod
     def getPropertyNames(cls, def_type: DefinitionPropertyType = None) -> List[str]:
         """Get the names of all supported properties.
-        
+
         :param type: :type{DefinitionPropertyType} The type of property to get the name of. Defaults to None which means all properties.
-        
+
         :return: A list of all the names of supported properties.
         """
 
@@ -473,9 +492,9 @@ class SettingDefinition:
     @classmethod
     def hasProperty(cls, name: str) -> bool:
         """Check if a property with the specified name is defined as a supported property.
-        
+
         :param name: :type{string} The name of the property to check if it is supported.
-        
+
         :return: True if the property is supported, False if not.
         """
 
@@ -484,9 +503,9 @@ class SettingDefinition:
     @classmethod
     def getPropertyType(cls, name: str) -> Optional[str]:
         """Get the type of a specified property.
-        
+
         :param name: :type{str} The name of the property to find the type of.
-        
+
         :return: DefinitionPropertyType corresponding to the type of the property or None if not found.
         """
 
@@ -498,12 +517,12 @@ class SettingDefinition:
     @classmethod
     def isRequiredProperty(cls, name: str) -> bool:
         """Check if the specified property is considered a required property.
-        
+
         Required properties are checked when deserializing a SettingDefinition and if not present an error
         will be reported.
-        
+
         :param name: :type{string} The name of the property to check if it is required or not.
-        
+
         :return: True if the property is supported and is required, False if it is not required or is not part of the
         list of supported properties.
         """
@@ -515,11 +534,11 @@ class SettingDefinition:
     @classmethod
     def isReadOnlyProperty(cls, name: str) -> bool:
         """Check if the specified property is considered a read-only property.
-        
+
         Read-only properties are properties that cannot have their value set in SettingInstance objects.
-        
+
         :param name: :type{string} The name of the property to check if it is read-only or not.
-        
+
         :return: True if the property is supported and is read-only, False if it is not required or is not part of the
         list of supported properties.
         """
@@ -531,12 +550,12 @@ class SettingDefinition:
     @classmethod
     def dependsOnProperty(cls, name: str) -> Optional[str]:
         """Check if the specified property depends on another property
-        
+
         The value of certain properties can change if the value of another property changes. This is used to signify
         that relation.
-        
+
         :param name: :type{string} The name of the property to check if it depends on another setting.
-        
+
         :return: :type{string} The property it depends on or None if it does not depend on another property.
         """
 
@@ -547,11 +566,11 @@ class SettingDefinition:
     @classmethod
     def addSettingType(cls, type_name: str, from_string: Optional[Callable[[str], Any]], to_string: Callable[[Any], str], validator: Optional[Validator] = None) -> None:
         """Add a new setting type to the list of accepted setting types.
-        
+
         :param type_name: The name of the new setting type.
         :param from_string: A function to call that converts to a proper value of this type from a string.
         :param to_string: A function that converts a value of this type to a string.
-        
+
         """
 
         cls.__type_definitions[type_name] = { "from": from_string, "to": to_string, "validator": validator }
@@ -559,12 +578,12 @@ class SettingDefinition:
     @classmethod
     def settingValueFromString(cls, type_name: str, string_value: str) -> Any:
         """Convert a string to a value according to a setting type.
-        
+
         :param type_name: :type{string} The name of the type to convert to.
         :param string_value: :type{string} The string to convert.
-        
+
         :return: The string converted to a proper value.
-        
+
         :exception ValueError: Raised when the specified type does not exist.
         """
 
@@ -580,12 +599,12 @@ class SettingDefinition:
     @classmethod
     def settingValueToString(cls, type_name: str, value: Any) -> str:
         """Convert a setting value to a string according to a setting type.
-        
+
         :param type_name: :type{string} The name of the type to convert from.
         :param value: The value to convert.
-        
+
         :return: :type{string} The specified value converted to a string.
-        
+
         :exception ValueError: Raised when the specified type does not exist.
         """
 
@@ -609,7 +628,7 @@ class SettingDefinition:
 
     def _deserialize_dict(self, serialized: Dict[str, Any]) -> None:
         """protected:
-    
+
         Deserialize from a dictionary
         """
 
