@@ -214,6 +214,23 @@ class TrustBasics:
             Logger.logException("e", "Save private/public key to '{0}','{1}' failed.".format(private_path, public_path))
         return False
 
+    @staticmethod
+    def removeCached(path: str) -> bool:
+        try:
+            cache_folders_to_empty = []  # List[str]
+            for root, dirnames, filenames in os.walk(path, followlinks=True):
+                for dirname in dirnames:
+                    if dirname == "__pycache__":
+                        cache_folders_to_empty.append(os.path.join(root, dirname))
+            for cache_folder in cache_folders_to_empty:
+                for root, dirnames, filenames in os.walk(cache_folder, followlinks=True):
+                    for filename in filenames:
+                        os.remove(os.path.join(root, filename))
+            return True
+        except:  # Yes, we  do really want this on _every_ exception that might occur.
+            Logger.logException("e", "Removal of pycache for unbundled path '{0}' failed.".format(path))
+        return False
+
 
 class Trust:
     """Trust for use in the main-application code, as opposed to the (keygen/signing) scripts.
