@@ -6,9 +6,11 @@ from typing import Optional
 
 from UM.Logger import Logger
 from UM.FileHandler.FileHandler import FileHandler
-from UM.FileHandler.FileReader import FileReader #For typing.
-from UM.FileHandler.ReadFileJob import ReadFileJob #For typing.
-from typing import TYPE_CHECKING
+from UM.FileHandler.ReadFileJob import ReadFileJob  # For typing.
+from typing import TYPE_CHECKING, cast
+
+from UM.Workspace.WorkspaceReader import WorkspaceReader
+
 if TYPE_CHECKING:
     from UM.Qt.QtApplication import QtApplication
 
@@ -16,19 +18,17 @@ if TYPE_CHECKING:
 ##  Central class for reading and writing workspaces.
 #   This class is created by Application and handles reading and writing workspace files.
 class WorkspaceFileHandler(FileHandler):
-
     def __init__(self, application: "QtApplication", writer_type: str = "workspace_writer", reader_type: str = "workspace_reader", parent: QObject = None) -> None:
         super().__init__(application, writer_type, reader_type, parent)
-        self.workspace_reader = None #type: Optional[FileReader]
+        self.workspace_reader = None  # type: Optional[WorkspaceReader]
 
-    def readerRead(self, reader: FileReader, file_name: str, **kwargs) -> Optional[str]:
+    def readerRead(self, reader: WorkspaceReader, file_name: str, **kwargs):
         self.workspace_reader = reader
         results = None
         try:
             results = reader.read(file_name)
-        except:
-            Logger.logException("e", "An exception occurred while loading workspace.")
-            Logger.log("w", "Unable to load workspace %s", file_name)
+        except Exception:
+            Logger.logException("w", "An exception occurred while loading workspace [%s]" % file_name)
 
         return results
 
@@ -40,18 +40,20 @@ class WorkspaceFileHandler(FileHandler):
 
     def _readWorkspaceFinished(self, job: ReadFileJob) -> None:
         # Add the loaded nodes to the scene.
-        nodes = job.getResult()
-        if nodes is not None:  # Job was not a failure.
-            # Delete all old nodes.
-            self._application.deleteAll()
-            # The name of the project is set after deleting all
-            self._application.workspaceLoaded.emit(job._filename)
+        result = job.getResult()
+        if isinstance(result, tuple):
+            nodes, metadata = result
+        else:
+            nodes = result
+            metadata = {}
 
-            # Add the loaded nodes to the scene.
-            nodes = job.getResult()
+        if nodes is not None:  # Job was not a failure.
+            self._application.resetWorkspace()
             for node in nodes:
                 # We need to prevent circular dependency, so do some just in time importing.
                 from UM.Operations.AddSceneNodeOperation import AddSceneNodeOperation
                 op = AddSceneNodeOperation(node, self._application.getController().getScene().getRoot())
                 op.push()
-                self._application.getController().getScene().sceneChanged.emit(node)
+
+            self._application.getWorkspaceMetadataStorage().setAllData(metadata)
+            self._application.workspaceLoaded.emit(cast(WorkspaceReader, self.workspace_reader).workspaceName())
